/*
 VARNA is a tool for the automated drawing, visualization and annotation of the secondary structure of RNA, designed as a companion software for web servers and databases.
 Copyright (C) 2008  Kevin Darty, Alain Denise and Yann Ponty.
 electronic mail : Yann.Ponty@lri.fr
 paper mail : LRI, bat 490 Universit Paris-Sud 91405 Orsay Cedex France

 This file is part of VARNA version 3.1.
 VARNA version 3.1 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

 VARNA version 3.1 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with VARNA version 3.1.
 If not, see http://www.gnu.org/licenses.
 */
package fr.orsay.lri.varna.models.rna;

import java.awt.geom.Point2D;

/**
 * The abstract rna base model
 * 
 * @author darty
 * 
 */
public abstract class ModeleBase {

	private ModeleStyleBP _styleBP;

	/**
	 * The base style.
	 */
	protected ModeleStyleBase _styleBase;

	/**
	 * TRUE if this InterfaceBase has to be colored, else FALSE.
	 */
	private Boolean _colorie;

	/**
	 * The coordinate representation of this InterfaceBase on the final graphic.
	 */
	private Point2D.Double _coords;

	/**
	 * The nearest loop center of this InterfaceBase.
	 */
	private Point2D.Double _center;

	/**
	 * The structure information assiociated with this InterfaceBase. It value
	 * is:<br>
	 * -1 if the structure element assiociated is a point or a dash.<br>
	 * If the structure assiociated is an opening bracket, it'll be the index of
	 * the closing bracket assiociated with this opening bracket.<br>
	 * If the structure assiociated is a closing bracket, it'll be the index of
	 * the opening bracket assiociated with this closing bracket.
	 */
	protected int _elementStructure;

	protected double _value;
	
	
	protected int _realIndex = -1;

	/**
	 * The internal index for this Base
	 */
	public abstract int getIndex();

	public abstract String getContent();

	/**
	 * Gets this InterfaceBase style.
	 * 
	 * @return this InterfaceBase style.
	 */
	public ModeleStyleBase get_styleBase() {
		return _styleBase;
	}

	public double getValue()
	{
		return _value;
	}

	public void setValue(double d)
	{
		_value = d;
	}
	
	
	/**
	 * Sets this InterfaceBase style.
	 * 
	 * @param base
	 *            - This InterfaceBase new style.
	 */
	public void set_styleBase(ModeleStyleBase base) {
		_styleBase = new ModeleStyleBase(base);
	}

	/**
	 * Gets this InterfaceBase color statement.
	 * 
	 * @return TRUE if this InterfaceBase has to be colored, else FALSE.
	 */
	public Boolean get_colorie() {
		return _colorie;
	}

	/**
	 * Sets this InterfaceBase color statement.
	 * 
	 * @param _colorie
	 *            - TRUE if you want this InterfaceBase to be colored, else
	 *            FALSE
	 */
	public void set_colorie(Boolean _colorie) {
		this._colorie = _colorie;
	}

	/**
	 * Gets this InterfaceBase coordinates on its current graphic.
	 * 
	 * @return this InterfaceBase coordinates on its current graphic.
	 */
	public Point2D.Double get_coords() {
		return _coords;
	}

	/**
	 * Sets this InterfaceBase coordinates on its current graphic.
	 * 
	 * @param _coords
	 *            - The new coordinates on current graphic of this
	 *            InterfaceBase.
	 */
	public void set_coords(Point2D.Double _coords) {
		this._coords = _coords;
	}

	/**
	 * Gets this InterfaceBase nearest loop center.
	 * 
	 * @return this InterfaceBase nearest loop center.
	 */
	public Point2D.Double get_center() {
		return _center;
	}

	/**
	 * Sets this InterfaceBase nearest loop center.
	 * 
	 * @param _center
	 *            - This InterfaceBase new nearest loop center
	 */
	public void set_center(Point2D.Double _center) {
		this._center = _center;
	}

	/**
	 * Gets this InterfaceBase associated structure element.
	 * 
	 * @return this InterfaceBase associated structure element.
	 */
	public int get_elementStructure() {
		return _elementStructure;
	}

	/**
	 * Sets this InterfaceBase assiocated structure element.
	 * 
	 * @param structure
	 *            - This new assiocated structure element.
	 */
	public void set_elementStructure(int structure) {
		set_elementStructure(structure, new ModeleStyleBP());
	}

	/**
	 * Sets this InterfaceBase associated structure element.
	 * 
	 * @param structure
	 *            - This new associated structure element.
	 * @param type
	 *            - The type of this base pair.
	 */
	public void set_elementStructure(int structure, ModeleStyleBP type) {
		_elementStructure = structure;
		_styleBP = type;

	}

	/**
	 * Gets the base pair type for this element.
	 * 
	 * @return the base pair type for this element.
	 */
	public ModeleStyleBP get_styleBP() {
		return _styleBP;
	}

	/**
	 * Sets the base pair type for this element.
	 * 
	 * @param type
	 *            - The new base pair type for this element.
	 */
	public void set_styleBP(ModeleStyleBP type) {
		_styleBP = type;
	}

	public int getBaseNumber() {
		return _realIndex;
	}

	public void setBaseNumber(int bn) {
		_realIndex = bn;
	}
	

}
